from datetime import datetime

from aiohttp import ClientSession

from .. import schemas
from ..enums import Format, Visibility
from ..exceptions import PaspybinBadAPIRequestError
from ..types import DevKey, PastebinUrl, PasteKey, UserKey
from .api import API

__all__ = ["Paste"]


class Paste(API, schemas.Paste):
    """
    Paste API wrapper.
    """

    def __init__(
        self,
        key: PasteKey,
        date: datetime,
        title: str,
        size: int,
        expire_date: datetime,
        private: Visibility,
        format: Format,
        url: PastebinUrl,
        hits: int,
        dev_key: DevKey | None = None,
        user_key: UserKey | None = None,
        session: ClientSession | None = None,
    ) -> None:
        API.__init__(self, dev_key, user_key, session)
        schemas.Paste.__init__(
            self, key, date, title, size, expire_date, private, format, url, hits
        )

    async def delete(self) -> None:
        """
        Delete a paste owned by user.

        Raises:
            PaspybinBadAPIRequestError: if a bad request is sent to the API.

        Examples:
            >>> import asyncio
            >>> import os
            >>> from paspybin import Paspybin
            >>> PASTEBIN_API_DEV_KEY = os.environ["PASTEBIN_API_DEV_KEY"]
            >>> PASTEBIN_USERNAME = os.environ["PASTEBIN_USERNAME"]
            >>> PASTEBIN_PASSWORD = os.environ["PASTEBIN_PASSWORD"]
            >>> async def main():
            ...     async with Paspybin(PASTEBIN_API_DEV_KEY) as paspybin:
            ...         await paspybin.login(PASTEBIN_USERNAME, PASTEBIN_PASSWORD)
            ...         async for paste in paspybin.pastes.get_all():
            ...             # paste.delete()
            ...             pass
            >>> asyncio.run(main())
        """
        payload = {
            "api_dev_key": self._dev_key,
            "api_option": "delete",
            "api_paste_key": self.key,
            "api_user_key": self._user_key,
        }

        async with self._session.post(self.api_post_url, data=payload) as response:
            data = await response.text()

            if not response.ok:
                raise PaspybinBadAPIRequestError(data)

    async def get_content(self) -> str:
        """
        Get the pasted content.

        Returns:
            A string of paste content.

        Raises:
            PaspybinBadAPIRequestError: if a bad request is sent to the API.

        Examples:
            >>> import asyncio
            >>> import os
            >>> from paspybin import Paspybin
            >>> PASTEBIN_API_DEV_KEY = os.environ["PASTEBIN_API_DEV_KEY"]
            >>> PASTEBIN_USERNAME = os.environ["PASTEBIN_USERNAME"]
            >>> PASTEBIN_PASSWORD = os.environ["PASTEBIN_PASSWORD"]
            >>> async def main():
            ...     async with Paspybin(PASTEBIN_API_DEV_KEY) as paspybin:
            ...         await paspybin.login(PASTEBIN_USERNAME, PASTEBIN_PASSWORD)
            ...         async for paste in paspybin.pastes.get_all():
            ...             paste_content = await paste.get_content()
            ...             # do what you want to do with paste content here
            >>> asyncio.run(main())
        """
        payload = {
            "api_dev_key": self._dev_key,
            "api_option": "show_paste",
            "api_paste_key": self.key,
            "api_user_key": self._user_key,
        }

        async with self._session.post(self.api_raw_url, data=payload) as response:
            data = await response.text()

            if not response.ok:
                raise PaspybinBadAPIRequestError(data)

        return data



"""
Module to store a str enum class representation the syntax highlighting format.
"""
from enum import StrEnum

__all__ = ["Format"]


class Format(StrEnum):
    """
    A str enum class that define valid syntax highlighting format.

    Attributes:
        NONE: `"text"`, None
        _4CS: `"4cs"`, 4CS
        _6502_ACME_CROSS_ASSEMBLER: `"6502acme"`, 6502 ACME Cross Assembler
        _6502_KICK_ASSEMBLER: `"6502kickass"`, 6502 Kick Assembler
        _6502_TASM_OR_64TASS: `"6502tasm"`, 6502 TASM/64TASS
        ABAP: `"abap"`, ABAP
        ACTIONSCRIPT: `"actionscript"`, ActionScript
        ACTIONSCRIPT3: `"actionscript3"`, ActionScript 3
        ADA: `"ada"`, Ada
        AIMMS: `"aimms"`, AIMMS
        ALGOL68: `"algol68"`, ALGOL 68
        APACHE_LOG: `"apache"`, Apache Log
        APPLESCRIPT: `"applescript"`, AppleScript
        APT_SOURCES: `"apt_sources"`, APT Sources
        ARDUINO: `"arduino"`, Arduino
        ARM: `"arm"`, ARM
        ASM: `"asm"`, ASM (NASM)
        ASP: `"asp"`, ASP
        ASYMPTOTE: `"asymptote"`, Asymptote
        AUTOCONF: `"autoconf"`, autoconf
        AUTOHOTKEY: `"autohotkey"`, Autohotkey
        AUTOIT: `"autoit"`, AutoIt
        AVISYNTH: `"avisynth"`, Avisynth
        AWK: `"awk"`, Awk
        BASCOM_AVR: `"bascomavr"`, BASCOM AVR
        BASH: `"bash"`, Bash
        BASIC4GL: `"basic4gl"`, Basic4GL
        BATCH: `"dos"`, Batch
        BIBTEX: `"bibtex"`, BibTeX
        BLITZ3D: `"b3d"`, Blitz3D
        BLITZ_BASIC: `"blitzbasic"`, Blitz Basic
        BLITZMAX: `"bmx"`, BlitzMax
        BNF: `"bnf"`, BNF
        BOO: `"boo"`, BOO
        BRAINFUCK: `"bf"`, BrainFuck
        C: `"c"`, C
        CSHARP: `"csharp"`, C#
        C_WINAPI: `"c_winapi"`, C (WinAPI)
        CPP: `"cpp"`, C++
        CPP_WINAPI: `"cpp_winapi"`, C++ (WinAPI)
        CPP_QT: `"cpp_qt"`, C++ (with Qt extensions
        C_LOADRUNNER: `"c_loadrunner"`, C: Loadrunner
        CAD_DCL: `"caddcl"`, CAD DCL
        CAD_LISP: `"cadlisp"`, CAD Lisp
        CEYLON: `"ceylon"`, Ceylon
        CFDG: `"cfdg"`, CFDG
        C_MACS: `"c_mac"`, C for Macs
        CHAISCRIPT: `"chaiscript"`, ChaiScript
        CHAPEL: `"chapel"`, Chapel
        C_INTERMEDIATE_LANGUAGE: `"cil"`, C Intermediate Language
        CLOJURE: `"clojure"`, Clojure
        CLONE_C: `"klonec"`, Clone C
        CLONE_CPP: `"klonecpp"`, Clone C++
        CMAKE: `"cmake"`, CMake
        COBOL: `"cobol"`, COBOL
        COFFEESCRIPT: `"coffeescript"`, CoffeeScript
        COLDFUSION: `"cfm"`, ColdFusion
        CSS: `"css"`, CSS
        CUESHEET: `"cuesheet"`, Cuesheet
        D: `"d"`, D
        DART: `"dart"`, Dart
        DCL: `"dcl"`, DCL
        DCPU16: `"dcpu16"`, DCPU-16
        DCS: `"dcs"`, DCS
        DELPHI: `"delphi"`, Delphi
        DELPHI_PRISM_OXYGENE: `"oxygene"`, Delphi Prism (Oxygene)
        DIFF: `"diff"`, Diff
        DIV: `"div"`, DIV
        DOT: `"dot"`, DOT
        E: `"e"`, E
        EASYTRIEVE: `"ezt"`, Easytrieve
        ECMASCRIPT: `"ecmascript"`, ECMAScript
        EIFFEL: `"eiffel"`, Eiffel
        EMAIL: `"email"`, Email
        EPC: `"epc"`, EPC
        ERLANG: `"erlang"`, Erlang
        EUPHORIA: `"euphoria"`, Euphoria
        FSHARP: `"fsharp"`, F#
        FALCON: `"falcon"`, Falcon
        FILEMAKER: `"filemaker"`, Filemaker
        FO_LANGUAGE: `"fo"`, FO Language
        FORMULA_ONE: `"f1"`, Formula One
        FORTRAN: `"fortran"`, Fortran
        FREEBASIC: `"freebasic"`, FreeBasic
        FREESWITCH: `"freeswitch"`, FreeSWITCH
        GAMBAS: `"gambas"`, GAMBAS
        GAME_MAKER: `"gml"`, Game Maker
        GDB: `"gdb"`, GDB
        GDSCRIPT: `"gdscript"`, GDScript
        GENERO: `"genero"`, Genero
        GENIE: `"genie"`, Genie
        GETTEXT: `"gettext"`, GetText
        GO: `"go"`, Go
        GODOT_GLSL: `"godot-glsl"`, Godot GLSL)
        GROOVY: `"groovy"`, Groovy
        GWBASIC: `"gwbasic"`, GwBasic
        HASKELL: `"haskell"`, Haskell
        HAXE: `"haxe"`, Haxe
        HICEST: `"hicest"`, HicEst
        HQ9_PLUS: `"hq9plus"`, HQ9 Plus
        HTML: `"html4strict"`, HTML
        HTML5: `"html5"`, HTML 5
        ICON: `"icon"`, Icon
        IDL: `"idl"`, IDL
        INI_FILE: `"ini"`, INI file
        INNO_SCRIPT: `"inno"`, Inno Script
        INTERCAL: `"intercal"`, INTERCAL
        IO: `"io"`, IO
        ISPF_PANEL_DEFINITION: `"ispfpanel"`, ISPF Panel Definition
        J: `"j"`, J
        JAVA: `"java"`, Java
        JAVA5: `"java5"`, Java 5
        JAVASCRIPT: `"javascript"`, JavaScript
        JCL: `"jcl"`, JCL
        JQUERY: `"jquery"`, jQuery
        JSON: `"json"`, JSON
        JULIA: `"julia"`, Julia
        KIXTART: `"kixtart"`, KiXtart
        KOTLIN: `"kotlin"`, Kotlin
        KSP_KONTAKT_SCRIPT: `"ksp"`, KSP (Kontakt Script)
        LATEX: `"latex"`, Latex
        LDIF: `"ldif"`, LDIF
        LIBERTY_BASIC: `"lb"`, Liberty BASIC
        LINDEN_SCRIPTING: `"lsl2"`, Linden Scripting
        LISP: `"lisp"`, Lisp
        LLVM: `"llvm"`, LLVM
        LOCO_BASIC: `"locobasic"`, Loco Basic
        LOGTALK: `"logtalk"`, Logtalk
        LOL_CODE: `"lolcode"`, LOL Code
        LOTUS_FORMULAS: `"lotusformulas"`, Lotus Formulas
        LOTUS_SCRIPT: `"lotusscript"`, Lotus Script
        LSCRIPT: `"lscript"`, LScript
        LUA: `"lua"`, Lua
        M68000_ASSEMBLER: `"m68k"`, M68000 Assembler
        MAGIKSF: `"magiksf"`, MagikSF
        MAKE: `"make"`, Make
        MAPBASIC: `"mapbasic"`, MapBasic
        MARKDOWN: `"markdown"`, Markdown
        MATLAB: `"matlab"`, MatLab
        MERCURY: `"mercury"`, Mercury
        METAPOST: `"metapost"`, MetaPost
        MIRC: `"mirc"`, mIRC
        MIX_ASSEMBLER: `"mmix"`, MIX Assembler
        MK_61_OR_52: `"MK-61/52"`, MK-61/52)/52)
        MODULA2: `"modula2"`, Modula 2
        MODULA3: `"modula3"`, Modula 3
        MOTOROLA_68000_HISOFT_DEV: `"68000devpac"`, Motorola 68000 HiSoft Dev
        MPASM: `"mpasm"`, MPASM
        MXML: `"mxml"`, MXML
        MYSQL: `"mysql"`, MySQL
        NAGIOS: `"nagios"`, Nagios
        NETREXX: `"netrexx"`, NetRexx
        NEWLISP: `"newlisp"`, newLISP
        NGINX: `"nginx"`, Nginx
        NIM: `"nim"`, Nim
        NULLSOFT_INSTALLER: `"nsis"`, NullSoft Installer
        OBERON2: `"oberon2"`, Oberon 2
        OBJECK_PROGRAMMING_LANGUA: `"objeck"`, Objeck Programming Langua
        OBJECTIVE_C: `"objc"`, Objective C
        OCAML: `"ocaml"`, OCaml
        OCAML_BRIEF: `"ocaml-brief"`, OCaml Brief)
        OCTAVE: `"octave"`, Octave
        OPENBSD_PACKET_FILTER: `"pf"`, OpenBSD PACKET FILTER
        OPENGL_SHADING: `"glsl"`, OpenGL Shading
        OPEN_OBJECT_REXX: `"oorexx"`, Open Object Rexx
        OPENOFFICE_BASIC: `"oobas"`, Openoffice BASIC
        ORACLE8: `"oracle8"`, Oracle 8
        ORACLE11: `"oracle11"`, Oracle 11
        OZ: `"oz"`, Oz
        PARASAIL: `"parasail"`, ParaSail
        PARI_GP: `"parigp"`, PARI/GP
        PASCAL: `"pascal"`, Pascal
        PAWN: `"pawn"`, Pawn
        PCRE: `"pcre"`, PCRE
        PER: `"per"`, Per
        PERL: `"perl"`, Perl
        PERL6: `"perl6"`, Perl 6
        PHIX: `"phix"`, Phix
        PHP: `"php"`, PHP
        PHP_BRIEF: `"php-brief"`, PHP Brief)
        PIC16: `"pic16"`, Pic 16
        PIKE: `"pike"`, Pike
        PIXEL_BENDER: `"pixelbender"`, Pixel Bender
        PL_I: `"pli"`, PL/I
        PL_SQL: `"plsql"`, PL/SQL
        POSTGRESQL: `"postgresql"`, PostgreSQL
        POSTSCRIPT: `"postscript"`, PostScript
        POV_RAY: `"povray"`, POV-Ray
        POWERBUILDER: `"powerbuilder"`, PowerBuilder
        POWERSHELL: `"powershell"`, PowerShell
        PROFTPD: `"proftpd"`, ProFTPd
        PROGRESS: `"progress"`, Progress
        PROLOG: `"prolog"`, Prolog
        PROPERTIES: `"properties"`, Properties
        PROVIDEX: `"providex"`, ProvideX
        PUPPET: `"puppet"`, Puppet
        PUREBASIC: `"purebasic"`, PureBasic
        PYCON: `"pycon"`, PyCon
        PYTHON: `"python"`, Python
        PYTHON_FOR_S60: `"pys60"`, Python for S60
        Q_KDBPLUS: `"q"`, q/kdb+
        QBASIC: `"qbasic"`, QBasic
        QML: `"qml"`, QML
        R: `"rsplus"`, R
        RACKET: `"racket"`, Racket
        RAILS: `"rails"`, Rails
        RBSCRIPT: `"rbs"`, RBScript
        REBOL: `"rebol"`, REBOL
        REG: `"reg"`, REG
        REXX: `"rexx"`, Rexx
        ROBOTS: `"robots"`, Robots
        ROFF_MANPAGE: `"roff"`, Roff Manpage
        RPM_SPEC: `"rpmspec"`, RPM Spec
        RUBY: `"ruby"`, Ruby
        RUBY_GNUPLOT: `"gnuplot"`, Ruby Gnuplot
        RUST: `"rust"`, Rust
        SAS: `"sas"`, SAS
        SCALA: `"scala"`, Scala
        SCHEME: `"scheme"`, Scheme
        SCILAB: `"scilab"`, Scilab
        SCL: `"scl"`, SCL
        SDLBASIC: `"sdlbasic"`, SdlBasic
        SMALLTALK: `"smalltalk"`, Smalltalk
        SMARTY: `"smarty"`, Smarty
        SPARK: `"spark"`, SPARK
        SPARQL: `"sparql"`, SPARQL
        SQF: `"sqf"`, SQF
        SQL: `"sql"`, SQL
        SSH_CONFIG: `"sshconfig"`, SSH Config
        STANDARDML: `"standardml"`, StandardML
        STONESCRIPT: `"stonescript"`, StoneScript
        SUPERCOLLIDER: `"sclang"`, SuperCollider
        SWIFT: `"swift"`, Swift
        SYSTEMVERILOG: `"systemverilog"`, SystemVerilog
        T_SQL: `"tsql"`, T-SQL
        TCL: `"tcl"`, TCL
        TERA_TERM: `"teraterm"`, Tera Term
        TEXGRAPH: `"texgraph"`, TeXgraph
        THINBASIC: `"thinbasic"`, thinBasic
        TYPESCRIPT: `"typescript"`, TypeScript
        TYPOSCRIPT: `"typoscript"`, TypoScript
        UNICON: `"unicon"`, Unicon
        UNREALSCRIPT: `"uscript"`, UnrealScript
        UPC: `"upc"`, UPC
        URBI: `"urbi"`, Urbi
        VALA: `"vala"`, Vala
        VBNET: `"vbnet"`, VB.NET
        VBSCRIPT: `"vbscript"`, VBScript
        VEDIT: `"vedit"`, Vedit
        VERILOG: `"verilog"`, VeriLog
        VHDL: `"vhdl"`, VHDL
        VIM: `"vim"`, VIM
        VISUALBASIC: `"vb"`, VisualBasic
        VISUALFOXPRO: `"visualfoxpro"`, VisualFoxPro
        VISUAL_PRO_LOG: `"visualprolog"`, Visual Pro Log
        WHITESPACE: `"whitespace"`, WhiteSpace
        WHOIS: `"whois"`, WHOIS
        WINBATCH: `"winbatch"`, Winbatch
        XBASIC: `"xbasic"`, XBasic
        XML: `"xml"`, XML
        XOJO: `"xojo"`, Xojo
        XORG_CONFIG: `"xorg_conf"`, Xorg Config
        XPP: `"xpp"`, XPP
        YAML: `"yaml"`, YAML
        YARA: `"yara"`, YARA
        Z80_ASSEMBLER: `"z80"`, Z80 Assembler
        ZXBASIC: `"zxbasic"`, ZXBasic

    Examples:
        >>> Format("text")
        <Format.NONE: 'text'>
        >>> Format["NONE"]
        <Format.NONE: 'text'>
        >>> Format.NONE
        <Format.NONE: 'text'>
        >>> Format.NONE == "text"
        True
        >>> print(Format.NONE)
        text

    Note:
        `NONE` is special format, as it name suggest it has no syntax highlighting.
    """

    NONE: str = "text"
    _4CS: str = "4cs"
    _6502_ACME_CROSS_ASSEMBLER: str = "6502acme"
    _6502_KICK_ASSEMBLER: str = "6502kickass"
    _6502_TASM_OR_64TASS: str = "6502tasm"
    ABAP: str = "abap"
    ACTIONSCRIPT: str = "actionscript"
    ACTIONSCRIPT3: str = "actionscript3"
    ADA: str = "ada"
    AIMMS: str = "aimms"
    ALGOL68: str = "algol68"
    APACHE_LOG: str = "apache"
    APPLESCRIPT: str = "applescript"
    APT_SOURCES: str = "apt_sources"
    ARDUINO: str = "arduino"
    ARM: str = "arm"
    ASM: str = "asm"
    ASP: str = "asp"
    ASYMPTOTE: str = "asymptote"
    AUTOCONF: str = "autoconf"
    AUTOHOTKEY: str = "autohotkey"
    AUTOIT: str = "autoit"
    AVISYNTH: str = "avisynth"
    AWK: str = "awk"
    BASCOM_AVR: str = "bascomavr"
    BASH: str = "bash"
    BASIC4GL: str = "basic4gl"
    BATCH: str = "dos"
    BIBTEX: str = "bibtex"
    BLITZ3D: str = "b3d"
    BLITZ_BASIC: str = "blitzbasic"
    BLITZMAX: str = "bmx"
    BNF: str = "bnf"
    BOO: str = "boo"
    BRAINFUCK: str = "bf"
    C: str = "c"
    CSHARP: str = "csharp"
    C_WINAPI: str = "c_winapi"
    CPP: str = "cpp"
    CPP_WINAPI: str = "cpp_winapi"
    CPP_QT: str = "cpp_qt"
    C_LOADRUNNER: str = "c_loadrunner"
    CAD_DCL: str = "caddcl"
    CAD_LISP: str = "cadlisp"
    CEYLON: str = "ceylon"
    CFDG: str = "cfdg"
    C_MACS: str = "c_mac"
    CHAISCRIPT: str = "chaiscript"
    CHAPEL: str = "chapel"
    C_INTERMEDIATE_LANGUAGE: str = "cil"
    CLOJURE: str = "clojure"
    CLONE_C: str = "klonec"
    CLONE_CPP: str = "klonecpp"
    CMAKE: str = "cmake"
    COBOL: str = "cobol"
    COFFEESCRIPT: str = "coffeescript"
    COLDFUSION: str = "cfm"
    CSS: str = "css"
    CUESHEET: str = "cuesheet"
    D: str = "d"
    DART: str = "dart"
    DCL: str = "dcl"
    DCPU16: str = "dcpu16"
    DCS: str = "dcs"
    DELPHI: str = "delphi"
    DELPHI_PRISM_OXYGENE: str = "oxygene"
    DIFF: str = "diff"
    DIV: str = "div"
    DOT: str = "dot"
    E: str = "e"
    EASYTRIEVE: str = "ezt"
    ECMASCRIPT: str = "ecmascript"
    EIFFEL: str = "eiffel"
    EMAIL: str = "email"
    EPC: str = "epc"
    ERLANG: str = "erlang"
    EUPHORIA: str = "euphoria"
    FSHARP: str = "fsharp"
    FALCON: str = "falcon"
    FILEMAKER: str = "filemaker"
    FO_LANGUAGE: str = "fo"
    FORMULA_ONE: str = "f1"
    FORTRAN: str = "fortran"
    FREEBASIC: str = "freebasic"
    FREESWITCH: str = "freeswitch"
    GAMBAS: str = "gambas"
    GAME_MAKER: str = "gml"
    GDB: str = "gdb"
    GDSCRIPT: str = "gdscript"
    GENERO: str = "genero"
    GENIE: str = "genie"
    GETTEXT: str = "gettext"
    GO: str = "go"
    GODOT_GLSL: str = "godot-glsl"
    GROOVY: str = "groovy"
    GWBASIC: str = "gwbasic"
    HASKELL: str = "haskell"
    HAXE: str = "haxe"
    HICEST: str = "hicest"
    HQ9_PLUS: str = "hq9plus"
    HTML: str = "html4strict"
    HTML5: str = "html5"
    ICON: str = "icon"
    IDL: str = "idl"
    INI_FILE: str = "ini"
    INNO_SCRIPT: str = "inno"
    INTERCAL: str = "intercal"
    IO: str = "io"
    ISPF_PANEL_DEFINITION: str = "ispfpanel"
    J: str = "j"
    JAVA: str = "java"
    JAVA5: str = "java5"
    JAVASCRIPT: str = "javascript"
    JCL: str = "jcl"
    JQUERY: str = "jquery"
    JSON: str = "json"
    JULIA: str = "julia"
    KIXTART: str = "kixtart"
    KOTLIN: str = "kotlin"
    KSP_KONTAKT_SCRIPT: str = "ksp"
    LATEX: str = "latex"
    LDIF: str = "ldif"
    LIBERTY_BASIC: str = "lb"
    LINDEN_SCRIPTING: str = "lsl2"
    LISP: str = "lisp"
    LLVM: str = "llvm"
    LOCO_BASIC: str = "locobasic"
    LOGTALK: str = "logtalk"
    LOL_CODE: str = "lolcode"
    LOTUS_FORMULAS: str = "lotusformulas"
    LOTUS_SCRIPT: str = "lotusscript"
    LSCRIPT: str = "lscript"
    LUA: str = "lua"
    M68000_ASSEMBLER: str = "m68k"
    MAGIKSF: str = "magiksf"
    MAKE: str = "make"
    MAPBASIC: str = "mapbasic"
    MARKDOWN: str = "markdown"
    MATLAB: str = "matlab"
    MERCURY: str = "mercury"
    METAPOST: str = "metapost"
    MIRC: str = "mirc"
    MIX_ASSEMBLER: str = "mmix"
    MK_61_OR_52: str = "MK-61/52"
    MODULA2: str = "modula2"
    MODULA3: str = "modula3"
    MOTOROLA_68000_HISOFT_DEV: str = "68000devpac"
    MPASM: str = "mpasm"
    MXML: str = "mxml"
    MYSQL: str = "mysql"
    NAGIOS: str = "nagios"
    NETREXX: str = "netrexx"
    NEWLISP: str = "newlisp"
    NGINX: str = "nginx"
    NIM: str = "nim"
    NULLSOFT_INSTALLER: str = "nsis"
    OBERON2: str = "oberon2"
    OBJECK_PROGRAMMING_LANGUA: str = "objeck"
    OBJECTIVE_C: str = "objc"
    OCAML: str = "ocaml"
    OCAML_BRIEF: str = "ocaml-brief"
    OCTAVE: str = "octave"
    OPENBSD_PACKET_FILTER: str = "pf"
    OPENGL_SHADING: str = "glsl"
    OPEN_OBJECT_REXX: str = "oorexx"
    OPENOFFICE_BASIC: str = "oobas"
    ORACLE8: str = "oracle8"
    ORACLE11: str = "oracle11"
    OZ: str = "oz"
    PARASAIL: str = "parasail"
    PARI_GP: str = "parigp"
    PASCAL: str = "pascal"
    PAWN: str = "pawn"
    PCRE: str = "pcre"
    PER: str = "per"
    PERL: str = "perl"
    PERL6: str = "perl6"
    PHIX: str = "phix"
    PHP: str = "php"
    PHP_BRIEF: str = "php-brief"
    PIC16: str = "pic16"
    PIKE: str = "pike"
    PIXEL_BENDER: str = "pixelbender"
    PL_I: str = "pli"
    PL_SQL: str = "plsql"
    POSTGRESQL: str = "postgresql"
    POSTSCRIPT: str = "postscript"
    POV_RAY: str = "povray"
    POWERBUILDER: str = "powerbuilder"
    POWERSHELL: str = "powershell"
    PROFTPD: str = "proftpd"
    PROGRESS: str = "progress"
    PROLOG: str = "prolog"
    PROPERTIES: str = "properties"
    PROVIDEX: str = "providex"
    PUPPET: str = "puppet"
    PUREBASIC: str = "purebasic"
    PYCON: str = "pycon"
    PYTHON: str = "python"
    PYTHON_FOR_S60: str = "pys60"
    Q_KDBPLUS: str = "q"
    QBASIC: str = "qbasic"
    QML: str = "qml"
    R: str = "rsplus"
    RACKET: str = "racket"
    RAILS: str = "rails"
    RBSCRIPT: str = "rbs"
    REBOL: str = "rebol"
    REG: str = "reg"
    REXX: str = "rexx"
    ROBOTS: str = "robots"
    ROFF_MANPAGE: str = "roff"
    RPM_SPEC: str = "rpmspec"
    RUBY: str = "ruby"
    RUBY_GNUPLOT: str = "gnuplot"
    RUST: str = "rust"
    SAS: str = "sas"
    SCALA: str = "scala"
    SCHEME: str = "scheme"
    SCILAB: str = "scilab"
    SCL: str = "scl"
    SDLBASIC: str = "sdlbasic"
    SMALLTALK: str = "smalltalk"
    SMARTY: str = "smarty"
    SPARK: str = "spark"
    SPARQL: str = "sparql"
    SQF: str = "sqf"
    SQL: str = "sql"
    SSH_CONFIG: str = "sshconfig"
    STANDARDML: str = "standardml"
    STONESCRIPT: str = "stonescript"
    SUPERCOLLIDER: str = "sclang"
    SWIFT: str = "swift"
    SYSTEMVERILOG: str = "systemverilog"
    T_SQL: str = "tsql"
    TCL: str = "tcl"
    TERA_TERM: str = "teraterm"
    TEXGRAPH: str = "texgraph"
    THINBASIC: str = "thinbasic"
    TYPESCRIPT: str = "typescript"
    TYPOSCRIPT: str = "typoscript"
    UNICON: str = "unicon"
    UNREALSCRIPT: str = "uscript"
    UPC: str = "upc"
    URBI: str = "urbi"
    VALA: str = "vala"
    VBNET: str = "vbnet"
    VBSCRIPT: str = "vbscript"
    VEDIT: str = "vedit"
    VERILOG: str = "verilog"
    VHDL: str = "vhdl"
    VIM: str = "vim"
    VISUALBASIC: str = "vb"
    VISUALFOXPRO: str = "visualfoxpro"
    VISUAL_PRO_LOG: str = "visualprolog"
    WHITESPACE: str = "whitespace"
    WHOIS: str = "whois"
    WINBATCH: str = "winbatch"
    XBASIC: str = "xbasic"
    XML: str = "xml"
    XOJO: str = "xojo"
    XORG_CONFIG: str = "xorg_conf"
    XPP: str = "xpp"
    YAML: str = "yaml"
    YARA: str = "yara"
    Z80_ASSEMBLER: str = "z80"
    ZXBASIC: str = "zxbasic"



"""
Module to store an int enum class representation account type.
"""
from enum import IntEnum

__all__ = ["Type"]


class Type(IntEnum):
    """
    An int enum class that define valid account type.

    Attributes:
        NORMAL: `0`
        PRO: `1`

    Examples:
        >>> Type(0)
        <Type.NORMAL: 0>
        >>> Type["NORMAL"]
        <Type.NORMAL: 0>
        >>> Type.NORMAL
        <Type.NORMAL: 0>
        >>> Type.NORMAL == 0
        True
        >>> print(Type.NORMAL)
        0
    """

    NORMAL: int = 0
    PRO: int = 1



from aiohttp import ClientSession

from ..types import DevKey, UserKey

__all__ = ["API"]


class API:
    """
    Base Pastebin API wrapper.

    This class has common attributes that all api must have.
    """

    base_url = "https://pastebin.com"

    api_url = "/api"
    raw_url = "/raw"

    api_post_url = api_url + "/api_post.php"
    api_login_url = api_url + "/api_login.php"
    api_raw_url = api_url + "/api_raw.php"

    def __init__(
        self,
        dev_key: DevKey | None = None,
        user_key: UserKey | None = None,
        session: ClientSession | None = None,
    ) -> None:
        self._dev_key = dev_key
        self._user_key = user_key
        self._session = session if session is not None else ClientSession(self.base_url)

    def is_authenticated(self) -> bool:
        return self._user_key is not None



from traceback import TracebackException
from types import TracebackType
from typing import Self

from ..exceptions import PaspybinBadAPIRequestError
from ..types import DevKey
from .api import API
from .pastes import Pastes
from .user import User

__all__ = [
    "Paspybin",
]


class Paspybin(API):
    """
    Main Pastebin API class.
    """

    def __init__(self, dev_key: DevKey | None = None):
        API.__init__(self, dev_key)

        self.user = User(self._dev_key, self._user_key, self._session)
        self.pastes = Pastes(self._dev_key, self._user_key, self._session)

    async def __aenter__(self) -> Self:
        return self

    async def __aexit__(
        self,
        exc_type: Exception,
        exc_val: TracebackException,
        traceback: TracebackType,
    ) -> None:
        await self.close()

    async def close(self) -> None:
        if self._session is not None:
            await self._session.close()

    async def login(self, username: str, password: str) -> None:
        """
        Authenticating to Pastebin API based on credential auth given.

        Args:
            username: username of user.
            password: password of user.

        Raises:
            PaspybinBadAPIRequestError: if a bad request is sent to the API.
            ValueError: if dev_key not supplied.

        Examples:
            >>> import asyncio
            >>> import os
            >>> PASTEBIN_API_DEV_KEY = os.environ["PASTEBIN_API_DEV_KEY"]
            >>> PASTEBIN_USERNAME = os.environ["PASTEBIN_USERNAME"]
            >>> PASTEBIN_PASSWORD = os.environ["PASTEBIN_PASSWORD"]
            >>> async def main():
            ...     async with Paspybin(PASTEBIN_API_DEV_KEY) as paspybin:
            ...         await paspybin.login(PASTEBIN_USERNAME, PASTEBIN_PASSWORD)
            >>> asyncio.run(main())
        """
        if self._dev_key is None:
            raise ValueError("dev_key is required to use this method")

        payload = {
            "api_dev_key": self._dev_key,
            "api_user_name": username,
            "api_user_password": password,
        }

        async with self._session.post(self.api_login_url, data=payload) as response:
            data = await response.text()

            if not response.ok:
                raise PaspybinBadAPIRequestError(data)

            user_key = data

            API.__init__(self, self._dev_key, user_key, self._session)

            self.user = User(self._dev_key, self._user_key, self._session)
            self.pastes = Pastes(self._dev_key, self._user_key, self._session)

    def logout(self) -> None:
        """
        Logging out the currently logged in user.

        Examples:
            >>> import asyncio
            >>> import os
            >>> PASTEBIN_API_DEV_KEY = os.environ["PASTEBIN_API_DEV_KEY"]
            >>> PASTEBIN_USERNAME = os.environ["PASTEBIN_USERNAME"]
            >>> PASTEBIN_PASSWORD = os.environ["PASTEBIN_PASSWORD"]
            >>> async def main():
            ...     async with Paspybin(PASTEBIN_API_DEV_KEY) as paspybin:
            ...         await paspybin.login(PASTEBIN_USERNAME, PASTEBIN_PASSWORD)
            ...         paspybin.logout()
            >>> asyncio.run(main())
        """
        if self._user_key is not None:
            user_key = None

            API.__init__(self, self._dev_key, user_key, self._session)

            self.user = User(self._dev_key, self._user_key, self._session)
            self.pastes = Pastes(self._dev_key, self._user_key, self._session)



__all__ = [
    "PaspybinError",
    "PaspybinBadAPIRequestError",
    "PaspybinNotFoundError",
    "PaspybinParseError",
]


class PaspybinError(Exception):
    """
    Paspybin general exception.
    """

    pass


class PaspybinBadAPIRequestError(PaspybinError):
    """
    Paspybin exception raised when a bad request occurs.
    """

    pass


class PaspybinNotFoundError(PaspybinError):
    """
    Paspybin exception raised when paste content is not found.
    """

    pass


class PaspybinParseError(PaspybinError):
    """
    Paspybin exception raised when the parser fails to parse.
    """

    pass



from dataclasses import dataclass

from ..enums import Expire, Format, Type, Visibility

__all__ = ["User"]


@dataclass(slots=True)
class User:
    """
    A schema used to store info about user.

    Attributes:
        name: name of user
        format: syntax highlighting format of user
        expiration: expire enum of user
        avatar_url: avatar url of user
        private: visibility of user
        website: website of user
        email: email of user
        location: location of user
        account_type: account type of user
    """

    name: str
    format: Format
    expiration: Expire
    avatar_url: str
    private: Visibility
    website: str | None
    email: str
    location: str | None
    account_type: Type



from http import HTTPStatus
from typing import Any, AsyncIterator

from ..enums import Expire, Format, Visibility
from ..exceptions import PaspybinBadAPIRequestError, PaspybinNotFoundError
from ..parsers import parse_pastes
from ..types import FolderKey, PasteKey
from .api import API
from .paste import Paste

__all__ = ["Pastes"]


class Pastes(API):
    async def get_all(self, limit: int | None = None) -> AsyncIterator[Paste]:
        """
        Get list of pastes owned by user.

        Args:
            limit: Length limit of the paste list. By default the limit is 50, the
                minimum limit is 1 and the maximum limit is 1000.

        Yields:
            Some pastes.

        Raises:
            PaspybinBadAPIRequestError: if a bad request is sent to the API.
            ValueError: if dev_key not supplied.
            ValueError: if guest use this method.
            ValueError: if the limit value is less than 1 or more than 1000.

        Examples:
            >>> import asyncio
            >>> import os
            >>> from paspybin import Paspybin
            >>> PASTEBIN_API_DEV_KEY = os.environ["PASTEBIN_API_DEV_KEY"]
            >>> PASTEBIN_USERNAME = os.environ["PASTEBIN_USERNAME"]
            >>> PASTEBIN_PASSWORD = os.environ["PASTEBIN_PASSWORD"]
            >>> async def main():
            ...     async with Paspybin(PASTEBIN_API_DEV_KEY) as paspybin:
            ...         await paspybin.login(PASTEBIN_USERNAME, PASTEBIN_PASSWORD)
            ...         async for paste in paspybin.pastes.get_all():
            ...             # do what you want to do with paste here
            ...             pass
            >>> asyncio.run(main())
        """
        if self._dev_key is None:
            raise ValueError("dev_key is required to use this method")

        if not self.is_authenticated():
            raise ValueError("only logged in users can access the paste list")

        payload: dict[str, Any] = {
            "api_dev_key": self._dev_key,
            "api_option": "list",
            "api_user_key": self._user_key,
        }

        if limit is not None:
            if limit < 1 or limit > 1000:
                raise ValueError("limit value must be between 1 and 1000")

            payload["api_results_limit"] = limit

        async with self._session.post(self.api_post_url, data=payload) as response:
            data = await response.text()

            if not response.ok:
                raise PaspybinBadAPIRequestError(data)
            else:
                for paste in parse_pastes(data):
                    yield Paste(
                        paste.key,
                        paste.date,
                        paste.title,
                        paste.size,
                        paste.expire_date,
                        paste.private,
                        paste.format,
                        paste.url,
                        paste.hits,
                        self._dev_key,
                        self._user_key,
                        self._session,
                    )

    async def get_content(self, paste_key: PasteKey) -> str:
        """
        Get the public or unlisted pasted content.

        Args:
            paste_key: A paste key of the paste content that you want to get.

        Returns:
            A string of paste content.

        Raises:
            PaspybinNotFoundError: if paste content not found.

        Examples:
            >>> import asyncio
            >>> from paspybin import Paspybin
            >>> async def main():
            ...     async with Paspybin() as paspybin:
            ...         paste_key = "0C343n0d"
            ...         paste_content = await paspybin.pastes.get_content(paste_key)
            ...         # do what you want to do with paste content here
            >>> asyncio.run(main())
        """
        async with self._session.get(f"{self.raw_url}/{paste_key}") as response:
            if response.status == HTTPStatus.NOT_FOUND:
                raise PaspybinNotFoundError("paste not found")
            data = await response.text()

        return data

    async def create_paste(
        self,
        content: str,
        title: str | None = None,
        format: Format | None = None,
        visibility: Visibility | None = None,
        expire: Expire | None = None,
        folder_key: FolderKey | None = None,
    ) -> PasteKey:
        """
        Create a new paste.

        Args:
            content: The paste content that you want to create.
            title: title of the paste.
            format: syntax highlighting format of the paste.
            visibility: visibility of the paste.
            expire: expire of the paste.
            folder_key: a folder that you want to store the paste.

        Returns:
            A paste key.

        Raises:
            PaspybinBadAPIRequestError: if a bad request is sent to the API.
            ValueError: if dev_key not supplied.
            ValueError: if the paste content is empty.
            ValueError: if the paste visibility is private for guest.

        Examples:
            >>> import asyncio
            >>> import os
            >>> from paspybin import Paspybin
            >>> PASTEBIN_API_DEV_KEY = os.environ["PASTEBIN_API_DEV_KEY"]
            >>> async def main():
            ...     async with Paspybin(PASTEBIN_API_DEV_KEY) as paspybin:
            ...         paste_content = "some paste content"
            ...         paste_key = await paspybin.pastes.create_paste(paste_content)
            ...         # do what you want to do with paste key here
            >>> asyncio.run(main())
        """
        if self._dev_key is None:
            raise ValueError("dev_key is required to use this method")

        if content == "":
            raise ValueError("paste content was empty")

        if visibility == Visibility.PRIVATE and not self.is_authenticated():
            raise ValueError("guest paste visibility cannot be private")

        payload = {
            "api_dev_key": self._dev_key,
            "api_option": "paste",
            "api_paste_code": content,
        }

        optional_payload: dict[str, Any] = {
            "api_user_key": self._user_key,
            "api_paste_name": title,
            "api_paste_format": format,
            "api_paste_private": visibility,
            "api_paste_expire_date": expire,
            "api_folder_key": folder_key,
        }

        payload.update({k: v for k, v in optional_payload.items() if v is not None})

        async with self._session.post(self.api_post_url, data=payload) as response:
            data = await response.text()

            if not response.ok:
                raise PaspybinBadAPIRequestError(data)

            return data[21:]



"""
Module to store an int enum class representation visibility.
"""
from enum import IntEnum

__all__ = ["Visibility"]


class Visibility(IntEnum):
    """
    An int enum class that define valid visibility.

    Attributes:
        PUBLIC: `0`
        UNLISTED: `1`
        PRIVATE: `2`

    Examples:
        >>> Visibility(0)
        <Visibility.PUBLIC: 0>
        >>> Visibility["PUBLIC"]
        <Visibility.PUBLIC: 0>
        >>> Visibility.PUBLIC
        <Visibility.PUBLIC: 0>
        >>> Visibility.PUBLIC == 0
        True
        >>> print(Visibility.PUBLIC)
        0
    """

    PUBLIC: int = 0
    UNLISTED: int = 1
    PRIVATE: int = 2



from dataclasses import dataclass
from datetime import datetime

from ..enums import Format, Visibility
from ..types import PastebinUrl, PasteKey

__all__ = ["Paste"]


@dataclass(slots=True)
class Paste:
    """
    A schema used to store info about paste.

    Attributes:
        key: key of paste
        date: created date of paste
        title: title of paste
        size: size of paste
        expire_date: expire date of paste
        private: visibility of paste
        format: syntax highlighting format of paste
        url: url location of paste
        hits: views count of paste
    """

    key: PasteKey
    date: datetime
    title: str | None
    size: int
    expire_date: datetime
    private: Visibility
    format: Format
    url: PastebinUrl
    hits: int



from .. import schemas
from ..exceptions import PaspybinBadAPIRequestError
from ..parsers import parse_user
from .api import API

__all__ = ["User"]


class User(API):
    async def get_detail(self) -> schemas.User:
        """
        Get a user detail.

        Returns:
            A `User` schema.

        Raises:
            PaspybinBadAPIRequestError: if a bad request is sent to the API.
            ValueError: if dev_key not supplied.
            ValueError: if guest use this method.

        Examples:
            >>> import asyncio
            >>> import os
            >>> from paspybin import Paspybin
            >>> PASTEBIN_API_DEV_KEY = os.environ["PASTEBIN_API_DEV_KEY"]
            >>> PASTEBIN_USERNAME = os.environ["PASTEBIN_USERNAME"]
            >>> PASTEBIN_PASSWORD = os.environ["PASTEBIN_PASSWORD"]
            >>> async def main():
            ...     async with Paspybin(PASTEBIN_API_DEV_KEY) as paspybin:
            ...         await paspybin.login(PASTEBIN_USERNAME, PASTEBIN_PASSWORD)
            ...         print(await paspybin.user.get_detail())
            >>> asyncio.run(main())
            User(...)
        """
        if self._dev_key is None:
            raise ValueError("dev_key is required to use this method")

        if not self.is_authenticated():
            raise ValueError("only logged in users can access user details")

        payload = {
            "api_dev_key": self._dev_key,
            "api_option": "userdetails",
            "api_user_key": self._user_key,
        }

        async with self._session.post(self.api_post_url, data=payload) as response:
            data = await response.text()

            if not response.ok:
                raise PaspybinBadAPIRequestError(data)
            else:
                return parse_user(data)



from collections.abc import Iterator
from datetime import datetime

# FIXME
# Element is used only for typing not parsing
from xml.etree.ElementTree import Element  # nosec B405

# FIXME
# remove `type: ignore` if there is a stub for defusedxml
from defusedxml.ElementTree import fromstring  # type: ignore

from .enums import Expire, Format, Type, Visibility
from .exceptions import PaspybinParseError
from .schemas import Paste, User

__all__ = [
    "parse_pastes",
    "parse_user",
]


def parse_pastes(data: str) -> Iterator[Paste]:
    """
    Parse pastes xml string.

    Args:
        data: str of pastes xml

    Yields:
        Some `Paste` schema.

    Raises:
        PaspybinParseError: if one or more required fields are not found.

    Examples:
    >>> data = (
    ...     "<paste>"
    ...     "<paste_key>0b42rwhf</paste_key>"
    ...     "<paste_date>1297953260</paste_date>"
    ...     "<paste_title>javascript test</paste_title>"
    ...     "<paste_size>15</paste_size>"
    ...     "<paste_expire_date>1297956860</paste_expire_date>"
    ...     "<paste_private>0</paste_private>"
    ...     "<paste_format_long>JavaScript</paste_format_long>"
    ...     "<paste_format_short>javascript</paste_format_short>"
    ...     "<paste_url>https://pastebin.com/0b42rwhf</paste_url>"
    ...     "<paste_hits>15</paste_hits>"
    ...     "</paste>"
    ... )
    >>> pastes = parse_pastes(data)
    >>> pastes
    <generator object parse_pastes at ...>
    """
    if data == "No pastes found.":
        return

    pastes: Element = fromstring(f"<pastes>{data}</pastes>")

    for paste in pastes:
        if len(paste) < 10:
            raise PaspybinParseError("one or more fields not found")

        paste_key = paste[0].text
        paste_date = paste[1].text
        paste_title = paste[2].text
        paste_size = paste[3].text
        paste_expire_date = paste[4].text
        paste_private = paste[5].text
        # paste_format_long = paste[6].text
        paste_format_short = paste[7].text
        paste_url = paste[8].text
        paste_hits = paste[9].text

        if paste_key is None:
            raise PaspybinParseError("paste_key value not found")
        elif paste_date is None:
            raise PaspybinParseError("paste_date value not found")
        elif paste_size is None:
            raise PaspybinParseError("paste_size value not found")
        elif paste_expire_date is None:
            raise PaspybinParseError("paste_expire_date value not found")
        elif paste_private is None:
            raise PaspybinParseError("paste_private value not found")
        elif paste_format_short is None:
            raise PaspybinParseError("paste_format_short value not found")
        elif paste_url is None:
            raise PaspybinParseError("paste_url value not found")
        elif paste_hits is None:
            raise PaspybinParseError("paste_hits value not found")

        yield Paste(
            paste_key,
            datetime.fromtimestamp(int(paste_date)),
            paste_title,
            int(paste_size),
            datetime.fromtimestamp(int(paste_expire_date)),
            Visibility(int(paste_private)),
            Format(paste_format_short),
            paste_url,
            int(paste_hits),
        )


def parse_user(data: str) -> User:
    """
    Parse user xml string.

    Args:
        data: str of user xml

    Returns:
        A `User` schema.

    Raises:
        PaspybinParseError: if one or more required fields are not found.

    Examples:
    >>> data = (
    ...     "<user>"
    ...     "<user_name>wiz_kitty</user_name>"
    ...     "<user_format_short>text</user_format_short>"
    ...     "<user_expiration>N</user_expiration>"
    ...     "<user_avatar_url>https://pastebin.com/cache/a/1.jpg</user_avatar_url>"
    ...     "<user_private>1</user_private>"
    ...     "<user_website>https://myawesomesite.com</user_website>"
    ...     "<user_email>oh@dear.com</user_email>"
    ...     "<user_location>New York</user_location>"
    ...     "<user_account_type>1</user_account_type>"
    ...     "</user>"
    ... )
    >>> user = parse_user(data)
    >>> user
    User(...)

    Note:
        `user_website` and `user_location` is optional.
    """
    user: Element = fromstring(data)

    if len(user) < 9:
        raise PaspybinParseError("one or more fields not found")

    user_name = user[0].text
    user_format_short = user[1].text
    user_expiration = user[2].text
    user_avatar_url = user[3].text
    user_private = user[4].text
    user_website = user[5].text
    user_email = user[6].text
    user_location = user[7].text
    user_account_type = user[8].text

    if user_name is None:
        raise PaspybinParseError("user_name value not found")
    elif user_format_short is None:
        raise PaspybinParseError("user_format_short value not found")
    elif user_expiration is None:
        raise PaspybinParseError("user_expiration value not found")
    elif user_avatar_url is None:
        raise PaspybinParseError("user_avatar_url value not found")
    elif user_private is None:
        raise PaspybinParseError("user_private value not found")
    elif user_email is None:
        raise PaspybinParseError("user_email value not found")
    elif user_account_type is None:
        raise PaspybinParseError("user_account_type value not found")

    return User(
        user_name,
        Format(user_format_short),
        Expire(user_expiration),
        user_avatar_url,
        Visibility(int(user_private)),
        user_website,
        user_email,
        user_location,
        Type(int(user_account_type)),
    )



"""
Module to store common enum classes used by pastebin.
"""
from .expire import Expire
from .format import Format
from .type import Type
from .visibility import Visibility

__all__ = [
    "Expire",
    "Format",
    "Type",
    "Visibility",
]



"""
Module to store a str enum class representation of expire date.
"""
from enum import StrEnum

__all__ = ["Expire"]


class Expire(StrEnum):
    """
    A str enum class that define valid expiration date.

    Attributes:
        NEVER: `"N"`, never expire
        TEN_MINUTES: `"10M"`, expire after ten minutes
        ONE_HOUR: `"1H"`, expire after one hour
        ONE_DAY: `"1D"`, expire after one day
        ONE_WEEK: `"1W"`, expire after one week
        TWO_WEEKS: `"2W"`, expire after two weeks
        ONE_MONTH: `"1M"`, expire after one month
        SIX_MONTHS: `"6M"`, expire after six month
        ONE_YEAR: `"1Y"`, expire after one year

    Examples:
        >>> Expire("N")
        <Expire.NEVER: 'N'>
        >>> Expire["NEVER"]
        <Expire.NEVER: 'N'>
        >>> Expire.NEVER
        <Expire.NEVER: 'N'>
        >>> Expire.NEVER == "N"
        True
        >>> print(Expire.NEVER)
        N
    """

    NEVER: str = "N"
    TEN_MINUTES: str = "10M"
    ONE_HOUR: str = "1H"
    ONE_DAY: str = "1D"
    ONE_WEEK: str = "1W"
    TWO_WEEKS: str = "2W"
    ONE_MONTH: str = "1M"
    SIX_MONTHS: str = "6M"
    ONE_YEAR: str = "1Y"
